__all__ = (
    "INVALID_YAMLS",
    "VALID_YAMLS",
    "YAML_FILES",
    "_is_nan",
    "normalize_yaml",
)

import math
from pathlib import Path
from typing import Any

import yaml_rs
from dirty_equals import IsFloatNan

# https://github.com/yaml/yaml-test-suite
YAML_TEST_SUITE = Path(__file__).resolve().parent / "data" / "yaml-test-suite"
YAML_FILES = list(YAML_TEST_SUITE.glob("*.yaml"))

ALL_YAMLS = 351


def _get_yamls():
    valid = []
    invalid = []
    skipped = []

    for yaml_file in YAML_FILES:
        docs = yaml_rs.loads(yaml_file.read_text(encoding="utf-8"), parse_datetime=False)
        docs = [docs] if isinstance(docs, dict) else docs

        has_fail = any(doc.get("fail", False) for doc in docs)
        has_skip = any(doc.get("skip", False) for doc in docs)

        if has_skip:
            skipped.append(yaml_file)
        elif has_fail:
            invalid.append(yaml_file)
        else:
            valid.append(yaml_file)

    return valid, invalid, skipped


VALID_YAMLS, INVALID_YAMLS, SKIPPED_YAMLS = _get_yamls()
assert (
    len(YAML_FILES)
    == len(VALID_YAMLS) + len(INVALID_YAMLS) + len(SKIPPED_YAMLS)
    == ALL_YAMLS
)
print(  # noqa: T201
    f"valid: {len(VALID_YAMLS)}\n"
    f"invalid: {len(INVALID_YAMLS)}\n"
    f"skipped: {len(SKIPPED_YAMLS)}\n",
)


def _is_nan(obj: Any) -> Any | dict[Any, Any] | list[Any] | IsFloatNan:
    if isinstance(obj, dict):
        return {k: _is_nan(v) for k, v in obj.items()}
    if isinstance(obj, list):
        return [_is_nan(v) for v in obj]
    if isinstance(obj, float) and math.isnan(obj):
        return IsFloatNan
    return obj


def normalize_yaml(doc: dict) -> Any:
    return (
        doc.get("yaml")
        .replace("␣", " ")
        .replace("»", "\t")
        .replace("—", "")  # Tab line continuation ——»
        .replace("←", "\r")
        .replace("⇔", "\ufeff")  # BOM character
        .replace("↵", "")  # Trailing newline marker
        .replace("∎\n", "")
    )



from pathlib import Path

import pytest
import yaml_rs

from tests import VALID_YAMLS, normalize_yaml


@pytest.mark.parametrize(
    ("v", "pattern"),
    [
        (
            type("_Class", (), {}),
            r"Cannot serialize <class 'type'> \(<class '.*_Class'>\) to YAML",
        ),
        (
            {"x": lambda x: x},
            r"Cannot serialize <class 'function'> \(<function <lambda> at 0x.*>\)",
        ),
        (
            {"x": 1 + 2j},
            r"Cannot serialize <class 'complex'> \(\(1\+2j\)\) to YAML",
        ),
        (
            {"valid": {"invalid": object()}},
            r"Cannot serialize <class 'object'> \(<object object at 0x.*>\) to YAML",
        ),
    ],
)
def test_incorrect_dumps(v, pattern):
    with pytest.raises(yaml_rs.YAMLEncodeError, match=pattern):
        yaml_rs.dumps(v)


@pytest.mark.parametrize("yaml", VALID_YAMLS)
def test_valid_yamls_dumps_from_test_suite(yaml: Path) -> None:
    load_from_str = yaml_rs.loads(yaml.read_text(encoding="utf-8"), parse_datetime=False)

    docs = [load_from_str] if isinstance(load_from_str, dict) else load_from_str

    for doc in docs:
        parsed_yaml = yaml_rs.loads(normalize_yaml(doc), parse_datetime=False)

        if isinstance(parsed_yaml, set):
            parsed_yaml = dict.fromkeys(parsed_yaml)

        dump = doc.get("dump")
        if dump is None:
            continue

        dumps = yaml_rs.dumps(parsed_yaml)
        # FIXME
        try:
            assert dumps == dump
        except AssertionError:
            pytest.skip(f"skip: {yaml}")



__all__ = (
    "YAMLDecodeError",
    "YAMLEncodeError",
    "__version__",
    "dump",
    "dumps",
    "load",
    "loads",
)

from pathlib import Path
from typing import Any, BinaryIO, TextIO

from ._yaml_rs import (
    YAMLDecodeError,
    YAMLEncodeError,
    _dumps,
    _loads,
    _version,
)

__version__: str = _version


def load(
    fp: BinaryIO,
    /,
    *,
    parse_datetime: bool = True,
) -> dict[str, Any] | list[dict[str, Any]]:
    _bytes = fp.read()
    try:
        _str = _bytes.decode()
    except AttributeError:
        msg = "File must be opened in binary mode, e.g. use `open('config.yaml', 'rb')`"
        raise TypeError(msg) from None
    return loads(_str, parse_datetime=parse_datetime)


def loads(
    s: str,
    /,
    *,
    parse_datetime: bool = True,
) -> dict[str, Any] | list[dict[str, Any]]:
    if not isinstance(s, str):
        raise TypeError(f"Expected str object, not '{type(s).__name__}'")
    return _loads(s, parse_datetime=parse_datetime)


def dump(obj: Any, /, file: str | Path | TextIO) -> int:
    _str = _dumps(obj)
    if isinstance(file, str):
        file = Path(file)
    if isinstance(file, Path):
        return file.write_text(_str, encoding="utf-8")
    else:
        return file.write(_str)


def dumps(obj: Any, /) -> str:
    return _dumps(obj)



import platform
import time
from collections.abc import Callable
from pathlib import Path

import altair as alt
import cpuinfo
import oyaml
import polars as pl
import ruamel.yaml
import ryaml
import strictyaml
import yaml as pyyaml
import yaml_rs

N = 2500


def benchmark(func: Callable, count: int) -> float:
    start = time.perf_counter()
    for _ in range(count):
        func()
    end = time.perf_counter()
    return end - start


def plot_benchmark(results: dict[str, float], save_path: Path) -> None:
    df = (
        pl.DataFrame({
            "parser": list(results.keys()),
            "exec_time": list(results.values()),
        })
        .sort("exec_time")
        .with_columns([
            (pl.col("exec_time") / pl.col("exec_time").min()).alias("slowdown"),
        ])
    )

    chart = (
        alt.Chart(df)
        .mark_bar(cornerRadiusTopLeft=6, cornerRadiusTopRight=6)
        .encode(
            x=alt.X("parser:N", sort=None, title="Parser", axis=alt.Axis(labelAngle=0)),
            y=alt.Y(
                "exec_time:Q",
                title="Execution Time (seconds, lower=better)",
                scale=alt.Scale(domain=(0, df["exec_time"].max() * 1.04)),
                axis=alt.Axis(grid=False),
            ),
            color=alt.Color("parser:N", legend=None, scale=alt.Scale(scheme="dark2")),
            tooltip=[
                alt.Tooltip("parser:N", title=""),
                alt.Tooltip("exec_time:Q", title="Execution Time (s)", format=".4f"),
                alt.Tooltip("slowdown:Q", title="Slowdown", format=".2f"),
            ],
        )
    )

    text = (
        chart.mark_text(
            align="center",
            baseline="bottom",
            dy=-2,
            fontSize=9,
            fontWeight="bold",
        )
        .transform_calculate(
            label='format(datum.exec_time, ".4f") + '
            '"s (x" + format(datum.slowdown, ".2f") + ")"',
        )
        .encode(text="label:N")
    )

    os = f"{platform.system()} {platform.release()}"
    cpu = cpuinfo.get_cpu_info()["brand_raw"]
    py = platform.python_version()
    (chart + text).properties(
        width=600,
        height=400,
        title={
            "text": "YAML parsers benchmark (loads)",
            "subtitle": f"Python: {py} ({os}) | CPU: {cpu}",
        },
    ).save(save_path)


file = Path(__file__).resolve().parent
bench_yaml = file.parent / "tests" / "data" / "bench.yaml"
data = bench_yaml.read_text(encoding="utf-8")


def run(run_count: int) -> None:
    loads = {
        "yaml_rs": lambda: yaml_rs.loads(data),
        "yaml_rs (parse_dt=False)": lambda: yaml_rs.loads(data, parse_datetime=False),
        "ryaml": lambda: ryaml.loads(data),
        "PyYAML": lambda: pyyaml.safe_load(data),
        "ruamel.yaml": lambda: ruamel.yaml.YAML(typ="safe").load(data),
        "oyaml": lambda: oyaml.safe_load(data),
        "strictyaml": lambda: strictyaml.load(data),
    }
    results = {name: benchmark(func, run_count) for name, func in loads.items()}
    plot_benchmark(results, save_path=file / "loads.svg")


if __name__ == "__main__":
    run(N)



from pathlib import Path
from typing import Any, BinaryIO, TextIO

__version__: str

def load(fp: BinaryIO, /, *, parse_datetime: bool = True) -> dict[str, Any]: ...
def loads(s: str, /, *, parse_datetime: bool = True) -> dict[str, Any]: ...
def dump(obj: Any, file: str | Path | TextIO) -> int: ...
def dumps(obj: Any) -> str: ...

class YAMLDecodeError(ValueError): ...
class YAMLEncodeError(TypeError): ...
